import { select,  
         map,
         csv,
         nest,
         ascending, 
         keys,
         tree,
         sum,
         hierarchy,
         stratify,
         linkHorizontal,
         zoom,
         event,
         scaleLinear,
         extent,
         max
             } 
from 'd3';


let dataset_gen = nest()
  .key( d => d.hotel)
  .key( d => d.country).sortKeys(d3.ascending)
  .key( d => d.reserved_room_type ).sortKeys(d3.ascending)
  .rollup( d => { return sum(d, d => d.lead_time );   } ); 

const client = map()
  .set("width", 1200 )
  .set("height",1200);
const margin = map()
  .set("width",50)
	.set("height",50)

const lead_tree = tree().size([client.get("height"),client.get("width")])

const svg = select("svg");

svg
  .attr('width',client.get('width'))
  .attr('height',client.get('height')) 

const g = svg.append('g');

svg.call(zoom().on('zoom',()=>{
		g.attr('transform',event.transform);
}));

const linkPathGenerator = linkHorizontal()
	.x( d => d.y)
	.y( d => d.x);

csv('population.csv').then( hotel => {
   
  hotel.forEach((h)=>{
     h.is_canceled =  +h.is_canceled;
     h.lead_time = +h.lead_time;
  }); 

var root_hotel = {}; 
 
dataset_gen.entries(hotel).forEach( (d) => {
   root_hotel.children = [];
   var hotel = {}
   hotel.id = d.key;
   hotel.children = []
 
   d.values.forEach( (d) => {
     
     var country = {};   
     country.id = d.key;
     country.children = [];
     
     var child_room = {};
        d.values.forEach( d => {
              var room = {};
              room.id = d.key;             
              room.children = [{"id":String(d.value)}];
              country.children.push(room);
        } );
      hotel.children.push(country); 
   })
   root_hotel.children.push(hotel);
}); 

const root = hierarchy(root_hotel);

const treeLayout = tree()
	.size([client.get("height")-margin.get("height"),client.get("width")-margin.get("width")])

const meta_data = treeLayout(root); 
  

g.selectAll('path').data(meta_data.links())
   .enter().append('path').attr('d',linkPathGenerator);
  
const font_func 
= (depth) => { return {1:'4em',2:'2em',3:'1.8em',4:'2em' }[depth];  };

console.log(meta_data.descendants());
  
const get_leaves = d=> (d.depth == 3)

console.log(meta_data.descendants().reduce(get_leaves))
  

  
  
g.selectAll('text').data(meta_data.descendants())
  .enter().append('text')
          .attr('x', d => d.y)
        	.attr('y', d => d.x)
          .text(d => { if(d.children) return d.data.id; } )
          .attr('font-size', d => font_func(d.depth))
          .attr('text-anchor', d => d.children ? 'middle' : 'start');
  
var rooms = meta_data.descendants().filter((d) => d.depth==4 ).map( d => d.data.id);
  
var circleColor = scaleLinear().domain([0,max(rooms)]).range(["green","red"]);
 
g.selectAll('circle').data(meta_data.descendants())
  .enter().append('circle')
  				.attr('cx',d => d.y)
          .attr('cy', d => d.x) 
          .attr('r',d => {if(!d.children) return 10;})
          .attr('fill',d => {if(!d.children) return circleColor(d.data.id) ;})
  


    
}  );
 
